import android.content.Context
import android.graphics.Color
import android.os.Handler
import android.os.Looper
import com.secrethero.secrethero.SecretHero
import com.secrethero.secrethero.SecretHeroListener
import com.secrethero.secrethero.config.SecretHeroConfig
import com.secrethero.secrethero.config.SecretHeroTheme
import com.secrethero.secrethero.data.network.SecretHeroEvent
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.MethodCall

class SecretHeroFlutter(
    private val channel: MethodChannel
) : MethodCallHandler, SecretHeroListener {

    lateinit var context: Context
    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "initialize" -> {
                val params = call.arguments as? Map<String, Any?>
                initialize(params)
            }

            "start" -> start()
            "release" -> SecretHero.release()
            else -> result.notImplemented()
        }
    }

    private fun start() {
        SecretHero.start(context)
    }

    private fun initialize(params: Map<String, Any?>?) {
        val token = params?.get("token") as? String ?: ""
        val userId = params?.get("userId") as? String ?: ""
        val displayName = params?.get("displayName") as? String ?: ""
        val theme = getTheme(params?.get("theme") as? Map<String, Any?>?)

        var configBuilder = SecretHeroConfig.Builder()
            .setUserId(userId)
            .setDisplayName(displayName)

        theme?.let {
            configBuilder = configBuilder.setTheme(it)
        }

        val config = configBuilder.build(token)
        SecretHero.initialize(config, listener = this)
    }

    private fun getTheme(params: Map<String, Any?>?): SecretHeroTheme? {
        if (params == null) return null

        return SecretHeroTheme.Builder()
            .setPrimaryColor(hexStringToColor(params["primary_color"] as? String))
            .setSecondaryColor(hexStringToColor(params["secondary_color"] as? String))
            .setHighlightColor(hexStringToColor(params["highlight_color"] as? String))
            .setButtonColor(hexStringToColor(params["button_color"] as? String))
            .build()
    }

    private fun sendEvent(event: String, params: Map<String, Any?>) {
        channel.invokeMethod(event, params)
    }

    override fun onInitialized() {
        Handler(Looper.getMainLooper()).post {
            sendEvent("onInitialized", emptyMap())
        }
    }

    override fun onInitializeFailed(errorMessage: String) {
        Handler(Looper.getMainLooper()).post {
            sendEvent("onInitializedFailed", mapOf("errorMessage" to errorMessage))
        }
    }

    override fun onOpen() {
        Handler(Looper.getMainLooper()).post {
            sendEvent("onOpen", emptyMap())
        }
    }

    override fun onClose() {
        Handler(Looper.getMainLooper()).post {
            sendEvent("onClose", emptyMap())
        }
    }

    override fun onEvent(event: SecretHeroEvent) {
        Handler(Looper.getMainLooper()).post {
            val eventName = when (event) {
                SecretHeroEvent.GAME_SUCCESS -> "game_success"
                SecretHeroEvent.GAME_OVER -> "game_over"
                SecretHeroEvent.GAME_START -> "game_start"
                else -> return@post
            }
            sendEvent("onEvent", mapOf("event" to eventName))
        }
    }

    private fun hexStringToColor(hexColor: String?): Int? {
        return try {
            Color.parseColor(hexColor)
        } catch (e: Exception) {
            null
        }
    }
}