import 'package:flutter/services.dart';

import 'secrethero_flutter_platform_interface.dart';

class SecretHeroConfig {
  final String token;
  final String userId;
  final String displayName;
  final SecretHeroTheme? theme;

  SecretHeroConfig({
    required this.token,
    required this.userId,
    required this.displayName, 
    this.theme,
  });
}

class SecretHeroTheme {
  final String? primaryColor;
  final String? secondaryColor;
  final String? highlightColor;
  final String? buttonColor;

  SecretHeroTheme({
    this.primaryColor,
    this.secondaryColor,
    this.highlightColor,
    this.buttonColor,
  });
}

enum SecretHeroMethods {
  initialize,
  start,
  release,
}

enum SecretHeroEvents {
  onInitialized,
  onInitializeFailed,
  onOpen,
  onClose,
  onEvent
}

class SecretHeroEventListener {
  void Function()? onInitialized;
  void Function(String)? onInitializeFailed;
  void Function()? onOpen;
  void Function()? onClose;
  void Function(String)? onEvent;

  SecretHeroEventListener({
    this.onInitialized,
    this.onInitializeFailed,
    this.onOpen,
    this.onClose,
    this.onEvent,
  });
}

class MethodChannelSecretheroFlutter extends SecretheroFlutterPlatform {

  SecretHeroEventListener? listener;

  @override
  Future<dynamic> invokedMethods(MethodCall call) async {
    SecretHeroEvents event = SecretHeroEvents.values.byName(call.method);
    switch (event) {
      case SecretHeroEvents.onInitialized:
        listener?.onInitialized?.call();
        break;
      case SecretHeroEvents.onInitializeFailed:
        listener?.onInitializeFailed?.call(call.arguments["errorMessage"]);
        break;
      case SecretHeroEvents.onOpen:
        listener?.onOpen?.call();
        break;
      case SecretHeroEvents.onClose:
        listener?.onClose?.call();
        break;
      case SecretHeroEvents.onEvent:
        listener?.onEvent?.call(call.arguments["event"]);
        break;
      default:
        print("invalid event");
        break;
    }
  }

  @override
  Future<void> start() async {
    await methodCall(
      SecretHeroMethods.start,
      {},
    );
  }

  @override
  Future<void> initialize(SecretHeroConfig config, SecretHeroEventListener listener) async {
    this.listener = listener;
    await methodCall(
      SecretHeroMethods.initialize,
      {
        "token": config.token,
        "userId": config.userId,
        "displayName": config.displayName,
        "theme": {
          "primary_color": config.theme?.primaryColor,
          "secondary_color": config.theme?.secondaryColor,
          "highlight_color": config.theme?.highlightColor,
          "button_color": config.theme?.buttonColor,
        }
      },
    );
  }

  @override
  Future<void> release() async {
    await methodCall(
      SecretHeroMethods.release,
      {},
    );
  }
}
