import 'package:flutter/services.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';

import 'secrethero_flutter_method_channel.dart';

abstract class SecretheroFlutterPlatform extends PlatformInterface {

  static final Object _token = Object();

  static SecretheroFlutterPlatform _instance = MethodChannelSecretheroFlutter();

  /// The default instance of [SecretheroFlutterPlatform] to use.
  ///
  /// Defaults to [MethodChannelSecretheroFlutter].
  static SecretheroFlutterPlatform get instance => _instance;

  /// Platform-specific implementations should set this with their own
  /// platform-specific class that extends [SecretheroFlutterPlatform] when
  /// they register themselves.
  static set instance(SecretheroFlutterPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  SecretheroFlutterPlatform() : super(token: _token) {
    methodChannel.setMethodCallHandler(invokedMethods);
  }

  final methodChannel = const MethodChannel('secrethero_flutter');

  Future<dynamic> invokedMethods(MethodCall call) async {
    throw UnimplementedError('invokedMethods() has not been implemented.');
  }

  Future start() {
    throw UnimplementedError('start() has not been implemented.');
  }

  Future initialize(SecretHeroConfig config, SecretHeroEventListener listener) {
    throw UnimplementedError('initialize() has not been implemented.');
  }

  Future release() {
    throw UnimplementedError('release() has not been implemented.');
  }

  Future methodCall(SecretHeroMethods event, Map<String, dynamic> map) async {
    await methodChannel.invokeMethod(
      event.name,
      map,
    );
  }
}
